@@ -1,6 +1,6 @@
 -- |
 -- Module      :  Text.Megaparsec.Stream
--- Copyright   :  © 2015–2019 Megaparsec contributors
+-- Copyright   :  © 2015–present Megaparsec contributors
 -- License     :  FreeBSD
 --
 -- Maintainer  :  Mark Karpov <markkarpov92@gmail.com>
@@ -14,7 +14,6 @@
 --
 -- @since 6.0.0
 
-{-# LANGUAGE CPP                 #-}
 {-# LANGUAGE FlexibleContexts    #-}
 {-# LANGUAGE FlexibleInstances   #-}
 {-# LANGUAGE LambdaCase          #-}
@@ -30,6 +29,7 @@ where
 
 import Data.Char (chr)
 import Data.Foldable (foldl')
+import Data.Kind (Type)
 import Data.List.NonEmpty (NonEmpty (..))
 import Data.Maybe (fromMaybe)
 import Data.Proxy
@@ -44,21 +44,17 @@ import qualified Data.List.NonEmpty         as NE
 import qualified Data.Text                  as T
 import qualified Data.Text.Lazy             as TL
 
-#if !MIN_VERSION_base(4,11,0)
-import Data.Semigroup
-#endif
-
 -- | Type class for inputs that can be consumed by the library.
 
 class (Ord (Token s), Ord (Tokens s)) => Stream s where
 
   -- | Type of token in the stream.
 
-  type Token s :: *
+  type Token s :: Type
 
   -- | Type of “chunk” of the stream.
 
-  type Tokens s :: *
+  type Tokens s :: Type
 
   -- | Lift a single token to chunk of the stream. The default
   -- implementation is:
@@ -136,12 +132,20 @@ class (Ord (Token s), Ord (Tokens s)) => Stream s where
 
   showTokens :: Proxy s -> NonEmpty (Token s) -> String
 
+  -- | Return the number of characters that a non-empty stream of tokens
+  -- spans. The default implementation is sufficient if every token spans
+  -- exactly 1 character.
+  --
+  -- @since 8.0.0
+
+  tokensLength :: Proxy s -> NonEmpty (Token s) -> Int
+  tokensLength Proxy = NE.length
+
   -- | Given an offset @o@ and initial 'PosState', adjust the state in such
   -- a way that it starts at the offset.
   --
-  -- Return three values (in order):
+  -- Return two values (in order):
   --
-  --     * 'SourcePos' which the given offset @o@ points to.
   --     * 'String' representing the line on which the given offset @o@ is
   --       located. The line should satisfy a number of conditions that are
   --       described below.
@@ -161,12 +165,15 @@ class (Ord (Token s), Ord (Tokens s)) => Stream s where
   --       spaces, which is determined by the 'pstateTabWidth' field of
   --       'PosState'.
   --
+  -- __Note__: type signature of the function was changed in the version
+  -- /8.0.0/.
+  --
   -- @since 7.0.0
 
   reachOffset
     :: Int             -- ^ Offset to reach
     -> PosState s      -- ^ Initial 'PosState' to use
-    -> (SourcePos, String, PosState s) -- ^ (See below)
+    -> (String, PosState s) -- ^ See the description of the function
 
   -- | A version of 'reachOffset' that may be faster because it doesn't need
   -- to fetch the line at which the given offset in located.
@@ -174,18 +181,19 @@ class (Ord (Token s), Ord (Tokens s)) => Stream s where
   -- The default implementation is this:
   --
   -- > reachOffsetNoLine o pst =
-  -- >   let (spos, _, pst')=  reachOffset o pst
-  -- >   in (spos, pst')
+  -- >   snd (reachOffset o pst)
+  --
+  -- __Note__: type signature of the function was changed in the version
+  -- /8.0.0/.
   --
   -- @since 7.0.0
 
   reachOffsetNoLine
     :: Int             -- ^ Offset to reach
     -> PosState s      -- ^ Initial 'PosState' to use
-    -> (SourcePos, PosState s) -- ^ Reached source position and updated state
+    -> PosState s      -- ^ Reached source position and updated state
   reachOffsetNoLine o pst =
-    let (spos, _, pst') = reachOffset o pst
-    in (spos, pst')
+    snd (reachOffset o pst)
 
 instance Stream String where
   type Token String = Char
@@ -320,9 +328,8 @@ reachOffset'
      -- ^ Offset to reach
   -> PosState s
      -- ^ Initial 'PosState' to use
-  -> (SourcePos, String, PosState s)
-     -- ^ Reached 'SourcePos', line at which 'SourcePos' is located, updated
-     -- 'PosState'
+  -> (String, PosState s)
+     -- ^ Line at which 'SourcePos' is located, updated 'PosState'
 reachOffset' splitAt'
              foldl''
              fromToks
@@ -330,8 +337,7 @@ reachOffset' splitAt'
              (newlineTok, tabTok)
              o
              PosState {..} =
-  ( spos
-  , case expandTab pstateTabWidth
+  ( case expandTab pstateTabWidth
            . addPrefix
            . f
            . fromToks
@@ -391,15 +397,14 @@ reachOffsetNoLine'
      -- ^ Offset to reach
   -> PosState s
      -- ^ Initial 'PosState' to use
-  -> (SourcePos, PosState s)
-     -- ^ Reached 'SourcePos' and updated 'PosState'
+  -> PosState s
+     -- ^ Updated 'PosState'
 reachOffsetNoLine' splitAt'
                    foldl''
                    (newlineTok, tabTok)
                    o
                    PosState {..} =
-  ( spos
-  , PosState
+  ( PosState
       { pstateInput = post
       , pstateOffset = max pstateOffset o
       , pstateSourcePos = spos
