@@ -1,3 +1,4 @@
+{-# LANGUAGE CPP               #-}
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE TypeFamilies      #-}
 
@@ -6,7 +7,6 @@ module Main (main) where
 import Control.DeepSeq
 import Control.Monad
 import Data.List.NonEmpty (NonEmpty (..))
-import Data.Semigroup ((<>))
 import Data.Text (Text)
 import Data.Void
 import Text.Megaparsec
@@ -17,6 +17,10 @@ import qualified Data.Set                   as E
 import qualified Data.Text                  as T
 import qualified Text.Megaparsec.Char.Lexer as L
 
+#if !MIN_VERSION_base(4,13,0)
+import Data.Semigroup ((<>))
+#endif
+
 -- | The type of parser that consumes 'String's.
 
 type Parser = Parsec Void Text
@@ -120,16 +124,15 @@ breachOffset o0 o1 = func
   f
   (o0 * 80, o1 * 80)
   where
-    f :: (Int, Int) -> (SourcePos, PosState Text)
+    f :: (Int, Int) -> PosState Text
     f (startOffset, targetOffset) =
-      let (x, _, y) = reachOffset targetOffset PosState
-            { pstateInput = manyAs (targetOffset - startOffset)
-            , pstateOffset = startOffset
-            , pstateSourcePos = initialPos ""
-            , pstateTabWidth = defaultTabWidth
-            , pstateLinePrefix = ""
-            }
-      in (x, y)
+      snd $ reachOffset targetOffset PosState
+        { pstateInput = manyAs (targetOffset - startOffset)
+        , pstateOffset = startOffset
+        , pstateSourcePos = initialPos ""
+        , pstateTabWidth = defaultTabWidth
+        , pstateLinePrefix = ""
+        }
 
 -- | Bench the 'reachOffsetNoLine' function.
 
@@ -142,7 +145,7 @@ breachOffsetNoLine o0 o1 = func
   f
   (o0 * 80, o1 * 80)
   where
-    f :: (Int, Int) -> (SourcePos, PosState Text)
+    f :: (Int, Int) -> PosState Text
     f (startOffset, targetOffset) =
       reachOffsetNoLine targetOffset PosState
         { pstateInput = manyAs (targetOffset - startOffset)
